import Vue from "vue";
import Router from "vue-router";
import { userInfo } from '@/stores/user'
import request from '@/utils/request';

const isAdmin = true
// const isAdmin = userInfo.user_role_id === '69'

Vue.use(Router);

const originalPush = Router.prototype.push;
Router.prototype.push = function (location) {
  originalPush.call(this, location).catch((err) => err);
};
const originalReplace = Router.prototype.replace;
Router.prototype.replace = function (location) {
  originalReplace.call(this, location).catch((err) => err);
};

/* AppLayout */
import AppLayout from "@/layout/app/index.vue";

/* MainDefaultLayout */
import MainDefaultLayout from "@/layout/main/default/index.vue";

/**
 * Note: sub-menu only appear when route children.length >= 1
 * Detail see: https://panjiachen.github.io/vue-element-admin-site/guide/essentials/router-and-nav
 *
 * hidden: true                   if set true, item will not show in the sidebar(default is false)
 * alwaysShow: true               if set true, will always show the root menu
 *                                if not set alwaysShow, when item has more than one children route,
 *                                it will becomes nested mode, otherwise not show the root menu
 * redirect: noRedirect           if set noRedirect will no redirect in the breadcrumb
 * name:'router-name'             the name is used by <keep-alive> (must set!!!)
 * meta : {
    roles: ['admin','editor']    control the page roles (you can set multiple roles)
    title: 'title'               the name show in sidebar and breadcrumb (recommend set)
    icon: 'svg-name'/'el-icon-x' the icon show in the sidebar
    breadcrumb: false            if set false, the item will hidden in breadcrumb(default is true)
    activeMenu: '/example/list'  if set path, the sidebar will highlight the path you set
  }
 */
import xxgzjcRoutes from "./routes/xxgzjc";
import xxbgOld from "./routes/xdbg_oldVersion";

/**
 * allRoutes
 * a base page that does not have permission requirements
 * all roles can be accessed
 */
export const allRoutes = [
  {
    path: "/jkdagl",
    component: AppLayout,
    redirect: "/jkdagl/sjgl",
    name: "jkdagl",
    meta: { title: "健康档案管理", icon: "el-icon-document" },
    children: [
      {
        path: "/jkdagl/sjgl",
        name: "jkdagl_sjgl",
        component: () => import("@/views/jkdagl/sjgl/index.vue"),
        meta: { title: "数据概览" },
      },
      {
        path: "xstzjkda",
        name: "jkdagl_xstzjkda",
        component: MainDefaultLayout,
        meta: { title: "学生体质健康档案" },
        children: [
          {
            path: "",
            name: "jkdagl_xstzjkda_index",
            component: () => import("@/views/jkdagl/xstzjkda/index.vue"),
            meta: { title: "学生体质健康档案" },
          },
          {
            path: "detail/:id",
            hidden: true,
            name: "xstzjkda_detail",
            component: () =>
              import("@/views/jkdagl/xstzjkda/detail/detail.vue"),
            meta: { title: "详情" },
            props: true,
          },
        ],
      },
      {
        path: "bbtj",
        name: "jkdagl_bbtj_main",
        component: MainDefaultLayout,
        children: [
          {
            path: "",
            name: "jkdagl_bbtj",
            component: () => import("@/views/jkdagl/bbtj/index.vue"),
            meta: { title: "报表统计" },
            props: true,
          },
          {
            path: "",
            name: "jkdagl_bbtj_detail",
            component: MainDefaultLayout,
            meta: { title: "报表详情" },
            hidden: true,
            children: [
              {
                path: "bbtj_detail/:jkbb_xq/:jkbb_bbmc",
                name: "bbtj_detail",
                component: () => import("@/views/jkdagl/bbtj/detail.vue"),
                meta: { title: "报表详情" },
              },
              {
                path: "bbtj_bbxq/:id",
                name: "bbtj_bbxq",
                component: () => import("@/views/jkdagl/bbtj/bbxq/index.vue"),
                meta: { title: "详情" },
                hidden: true,
              },
            ],
          },
        ],
      },
    ],
  },
  {
    path: "/gwglxt",
    name: "gwglxt",
    component: AppLayout,
    meta: { title: "公文收发管理", icon: "el-icon-document" },
    redirect: "/gwglxt/fwng",
    children: [
      {
        path: "/gwglxt/fwng",
        name: "gwglxt_fwng",
        component: () => import("@/views/gwglxt/fwng/index.vue"),
        meta: { title: "发文拟稿" },
      },
      {
        path: "/gwglxt/wdfw",
        name: "gwglxt_wdfw",
        component: () => import("@/views/gwglxt/wdfw/index.vue"),
        meta: { title: "我的发文" },
      },
      {
        path: "/gwglxt/dbfw",
        name: "gwglxt_dbfw_main",
        component: MainDefaultLayout,
        meta: { title: "待办发文" },
        children: [
          {
            path: "",
            name: "gwglxt_dbfw",
            component: () => import("@/views/gwglxt/dbfw/index.vue"),
            meta: { title: "待办发文" },
            props: true,
          },
          {
            path: "sh/:id",
            hidden: true,
            name: "gwglxt_dbfw_sh",
            component: () => import("@/views/gwglxt/dbfw/sh/index.vue"),
            meta: { title: "审核" },
            props: true,
          },
          {
            path: "lsbjjl/:id",
            hidden: true,
            name: "gwglxt_dbfw_lsbjjl",
            component: () => import("@/views/gwglxt/dbfw/lsbjjl/index.vue"),
            meta: { title: "历史编辑记录" },
            props: true,
          },
          {
            path: "dzqz/:id",
            hidden: true,
            name: "gwglxt_dbfw_dzqz",
            component: () => import("@/views/gwglxt/dbfw/dzqz/index.vue"),
            meta: { title: "电子签章" },
            props: true,
          },
        ],
      },
      {
        path: "/gwglxt/gwjh",
        name: "gwglxt_gwjh",
        component: () => import("@/views/gwglxt/gwjh/index.vue"),
        meta: { title: "公文交互" },
      },
      {
        path: "/gwglxt/dygw",
        name: "gwglxt_dygw",
        component: () => import("@/views/gwglxt/dygw/index.vue"),
        meta: { title: "待阅公文" },
      },
      {
        path: "/gwglxt/gwk",
        name: "gwglxt_gwk_main",
        component: MainDefaultLayout,
        meta: { title: "公文库" },
        children: [
          {
            path: "",
            name: "gwglxt_gwk",
            component: () => import("@/views/gwglxt/gwk/index.vue"),
            meta: { title: "公文库" },
            props: true,
          },
          {
            path: "gwxq/:id",
            hidden: true,
            name: "gwglxt_dbfw_gwxq",
            component: () => import("@/views/gwglxt/gwk/gwxq/index.vue"),
            meta: { title: "公文详情" },
            props: true,
          },
        ],
      },
      {
        path: "/gwglxt/lcgl",
        name: "gwglxt_lcgl",
        component: () => import("@/views/gwglxt/lcgl/index.vue"),
        meta: { title: "流程管理" },
      },
    ],
  },
  {
    path: "/xsjsgl",
    component: AppLayout,
    redirect: "/xsjsgl/cjbs",
    name: "xsjsgl",
    meta: { title: "学生竞赛管理", icon: "el-icon-document" },
    children: [
      {
        path: "cjbs",
        name: "xsjsgl_cjbs",
        component: MainDefaultLayout,
        meta: { title: "创建比赛" },
        children: [
          {
            path: "",
            name: "xsjsgl_cjbs_index",
            component: () => import("@/views/xsjsgl/cjbs/index.vue"),
            meta: { title: "创建比赛" },
          },
          {
            path: "detail/:id",
            hidden: true,
            name: "cjbs_detail",
            component: () => import("@/views/xsjsgl/cjbs/detail/detail.vue"),
            meta: { title: "详情" },
            props: true,
          },
          {
            path: "ryDetail/:id",
            hidden: true,
            name: "cjbs_ryDetail",
            component: () => import("@/views/xsjsgl/cjbs/detail/ryDetail.vue"),
            meta: { title: "详情" },
            props: true,
          },
        ],
      },

      {
        path: "zsdy",
        name: "xsjsgl_zsdy",
        component: () => import("@/views/xsjsgl/zsdy/index.vue"),
        meta: { title: "证书打印" },
      },
      {
        path: "sjhz",
        name: "xsjsgl_sjhz",
        component: MainDefaultLayout,
        meta: { title: "数据汇总" },
        children: [
          {
            path: "sjhz",
            name: "xsjsgl_sjhz_index",
            component: () => import("@/views/xsjsgl/sjhz/index.vue"),
            meta: { title: "数据汇总" },
          },
          {
            path: "detail/:bb_id",
            hidden: true,
            name: "sjhz_detail",
            component: () => import("@/views/xsjsgl/sjhz/detail/detail.vue"),
            meta: { title: "报表详情" },
            props: true,
          },
        ],
      },

      // {
      //     path: "rylb",
      //     name: "xsjsgl_rylb",
      //     component: () => import("@/views/xsjsgl/rylb/index.vue"),
      //     meta: {title: "荣誉列表"},
      // },
    ],
  },
  {
    path: "/jsld",
    component: AppLayout,
    redirect: "/jsld/xxbwrygl",
    name: "jsld",
    meta: { title: "教师流动管理", icon: "el-icon-document" },
    children: [
      {
        path: "xxbwrygl",
        name: "jsld_xxbwrygl_main",
        component: MainDefaultLayout,
        meta: { title: "学校编外人员管理" },
        children: [
          {
            path: "",
            name: "jsld_xxbwrygl",
            component: () => import("@/views/jsld/xxbwrygl/index.vue"),
            meta: { title: "学校编外人员管理" },
            props: true,
          },
          {
            path: "detail/:id",
            hidden: true,
            name: "jsld_xxbwrygl_detail",
            component: () => import("@/views/jsld/xxbwrygl/detail/index.vue"),
            meta: { title: "详情" },
            props: true,
          },
        ],
      },
      {
        path: "ndkhhz",
        name: "jsld_ndkhhz",
        component: () => import("@/views/jsld/ndkhhz/index.vue"),
        meta: { title: "年度考核汇总" },
      },

      {
        path: "myjsqkhz",
        name: "jsld_myjsqkhz",
        component: () => import("@/views/jsld/myjsqkhz/index.vue"),
        meta: { title: "名优教师情况汇总" },
      },

      {
        path: "txjstx",
        name: "jsld_txjstx",
        component: () => import("@/views/jsld/txjstx/index.vue"),
        meta: { title: "退休教师提醒" },
      },
      {
        path: "myjsjxkh",
        name: "jsld_myjsjxkh",
        component: MainDefaultLayout,
        meta: { title: "名优教师绩效考核" },
        children: [
          {
            path: "lbgl",
            name: "myjsjxkh_wsqd",
            component: MainDefaultLayout,
            props: true,
            meta: { title: "我申请的" },
            children: [
              {
                path: "",
                name: "jsld_myjsjxkh_wsqd_list",
                component: () => import("@/views/jsld/myjsjxkh/lbgl/index.vue"),
                meta: { title: "我申请的" },
                props: true,
              },
              {
                path: "detail/:id",
                name: "jsld_myjsjxkh_wsqd_detail",
                component: () => import("@/views/jsld/myjsjxkh/lbgl/xq/index.vue"),
                meta: { title: "详情" },
                props: true,
                hidden: true,
              },
            ],
          },

          {
            path: "wshd",
            name: "myjsjxkh_wshd",
            component: MainDefaultLayout,
            meta: { title: "我审核的" },
            props: true,
            children: [
              {
                path: "",
                name: "jsld_myjsjxkh_wshd",
                component: () => import("@/views/jsld/myjsjxkh/wshd/index.vue"),
                meta: { title: "我审核的" },
                props: true,
              },
              {
                path: "detail/:id",
                hidden: true,
                name: "jsld_myjsjxkh_wshd_detail",
                component: () =>
                  import("@/views/jsld/myjsjxkh/wshd/detail/index.vue"),
                meta: { title: "详情" },
                props: true,
              },
            ],
          },
          {
            path: "khhz",
            name: "myjsjxkh_khhz",
            component: MainDefaultLayout,
            props: true,
            meta: { title: "考核汇总" },
            children: [
              {
                path: "",
                name: "jsld_myjsjxkh_khhz",
                component: () => import("@/views/jsld/myjsjxkh/khhz/index.vue"),
                meta: { title: "考核汇总" },
                props: true,
              },
              {
                path: "detail/:id",
                hidden: true,
                name: "jsld_myjsjxkh_khhz_detail",
                component: () =>
                  import("@/views/jsld/myjsjxkh/khhz/detail/index.vue"),
                meta: { title: "详情" },
                props: true,
              },
            ],
          },
        ],
      },
      // {
      //   path: "bbtj",
      //   component: MainDefaultLayout,
      //   meta: { title: "报表统计" },
      //   children: [
      //     {
      //       path: "",
      //       name: "jsld_bbtj",
      //       component: () => import("@/views/jsld/bbtj/index.vue"),
      //       meta: { title: "报表统计" },
      //       props: true,
      //     },
      //     {
      //       path: "bbxq/:id",
      //       hidden: true,
      //       name: "jsld_bbtj_bbxq",
      //       component: () => import("@/views/jsld/bbtj/bbxq/index.vue"),
      //       meta: { title: "报表详情" },
      //       props: true,
      //     },
      //   ],
      // },
    ],
  },
  {
    path: "/dddx",
    component: AppLayout,
    redirect: "/dddx/ddrw",
    name: "dddx",
    meta: { title: "督导督学系统", icon: "el-icon-document" },
    children: [
      // {
      //   path: "ddrw",
      //   name: "dddx-ddrw",
      //   component: () => import("@/views/dddx/ddrw/index.vue"),
      //   redirect: "/dddx/ddrw/ddrw-main",
      //   meta: { title: "督导任务" },
      //   children: [
      //     {
      //       hidden: true,
      //       path: "ddrw-main",
      //       name: "dddx-ddrw-main",
      //       component: () => import("@/views/dddx/ddrw/components/ddrw.vue"),
      //       meta: { title: "" },
      //     },
      //     {
      //       hidden: true,
      //       path: "ddrw-tjzb/:id",
      //       name: "dddx-ddrw-tjzb",
      //       component: () => import("@/views/dddx/ddrw/components/tjzb.vue"),
      //       meta: { title: "添加指标" },
      //     },
      //     {
      //       hidden: true,
      //       path: "ddrw-detail/:id",
      //       name: "dddx-ddrw-detail",
      //       component: () => import("@/views/dddx/ddrw/components/detail.vue"),
      //       meta: { title: "详情" },
      //     },
      //   ],
      // },

      {
        path: "ddrw",
        name: "dddx-ddrw",
        component: () => import("@/views/dddx/ddrw/index.vue"),
        redirect: "/dddx/ddrw/ddrw-main",
        meta: { title: "督导任务" },
        children: [
          {
            hidden: true,
            path: "ddrw-main",
            name: "dddx-ddrw-main",
            component: () => import("@/views/dddx/ddrw/components/ddrw.vue"),
            meta: { title: "" },
          },
          {
            hidden: true,
            path: "ddrw-tjzb/:id",
            name: "dddx-ddrw-tjzb",
            component: () => import("@/views/dddx/ddrw/components/tjzb.vue"),
            meta: { title: "添加指标" },
          },
          {
            hidden: true,
            path: "ddrw-detail",
            name: "dddx-ddrw-detail",
            component: () =>
              import("@/views/dddx/ddrw/components/detail/index.vue"),
            meta: { title: "" },
            children: [
              {
                hidden: true,
                path: "ddrw-detail-index/:id",
                name: "dddx-ddrw-detail-index",
                component: () =>
                  import("@/views/dddx/ddrw/components/detail/ddrw-detail.vue"),
                meta: { title: "详情" },
              },
              {
                hidden: true,
                path: "indicator-detail/:id/:xfId",
                name: "dddx-ddrw-indicator-detail",
                component: () =>
                  import(
                    "@/views/dddx/ddrw/components/detail/indicator-detail.vue"
                  ),
                meta: { title: "详情" },
              },
              {
                hidden: true,
                path: "noindicator-detail/:id/:xfId",
                name: "dddx-ddrw-noindicator-detail",
                component: () =>
                  import(
                    "@/views/dddx/ddrw/components/detail/noindicator-detail.vue"
                  ),
                meta: { title: "上传" },
              },
            ],
          },
        ],
      },

      {
        path: "ddrwfk",
        name: "dddx_ddrwfk_main",
        component: MainDefaultLayout,
        meta: { title: "督导任务反馈" },
        children: [
          {
            path: "",
            name: "dddx_ddrwfk",
            component: () => import("@/views/dddx/ddrwfk/index.vue"),
            meta: { title: "督导任务反馈" },
          },
          {
            path: "detail/:id",
            name: "ddrwfk_detail",
            hidden: true,
            component: () => import("@/views/dddx/ddrwfk/detail/index.vue"),
            meta: { title: "反馈详情" },
          },
        ],
      },
      {
        path: "ddzg",
        name: "dddx_ddzg_main",
        component: MainDefaultLayout,
        meta: { title: "督导整改" },
        children: [
          {
            path: "",
            name: "dddx_ddzg",
            meta: { title: "督导整改" },
            component: () => import("@/views/dddx/ddzg/index.vue"),
          },
          {
            path: "ddzg_detail/:id",
            name: "ddzg_detail",
            hidden: true,
            component: () => import("@/views/dddx/ddzg/detail.vue"),
            meta: { title: "详情" },
          },
        ],
      },
      {
        path: "ddpx",
        name: "dddx_ddpx_main",
        component: MainDefaultLayout,
        meta: { title: "督导培训" },
        children: [
          {
            path: "",
            name: "dddx_ddpx",
            meta: { title: "督导培训" },
            component: () => import("@/views/dddx/ddpx/index.vue"),
          },
          {
            path: "ddpx_detail/:id",
            name: "ddpx_detail",
            hidden: true,
            component: () => import("@/views/dddx/ddpx/detail.vue"),
            meta: { title: "详情" },
          },
        ],
      },
      // {
      //     path: "",
      //     name: "ddzs_index",
      //     component: () => import("@/views/dddx/ddzs/index.vue"),
      //     meta: { title: "督导综述" },
      // },
      // {
      //   path: "jyjdetail/:id",
      //   hidden: true,
      //   name: "dddx_jyj_detail",
      //   component: () => import("@/views/dddx/ddzs/detail/jyjDetail.vue"),
      //   meta: { title: "详情" },
      //   props: true,
      // },
      // {
      //   path: "yzbdetail/:id",
      //   hidden: true,
      //   name: "dddx_xxyzb_detail",
      //   component: () => import("@/views/dddx/ddzs/detail/xxyzbDetail.vue"),
      //   meta: { title: "详情" },
      //   props: true,
      // },
      // {
      //   path: "wzbdetail/:id",
      //   hidden: true,
      //   name: "dddx_xxwzb_detail",
      //   component: () => import("@/views/dddx/ddzs/detail/xxwzbDetail.vue"),
      //   meta: { title: "详情" },
      //   props: true,
      // },
      // {
      //   path: "ddgzjh/:id",
      //   name: "dddx_ddgzjh",
      //   component: () => import("@/views/dddx/ddgzjh/index.vue"),
      //   meta: { title: "督导工作计划" },
      //   props: true,
      // },
      // {
      //   path: "ddgzjl",
      //   name: "dddx_ddgzjl",
      //   component: () => import("@/views/dddx/ddgzjl/index.vue"),
      //   meta: { title: "督导工作记录" },
      //   props: true,
      // },
      // {
      //   path: "ddqkfk",
      //   name: 'dddx_ddqkfk',
      //   component: () => import("@/views/dddx/ddqkfk/index.vue"),
      //   meta: { title: "督导情况反馈" },
      // },
      // {
      //   path: "ddzs",
      //   name: "ddzs",
      //   component: MainDefaultLayout,
      //   meta: { title: "督导综述" },
      //   children: [
      //     {
      //       path: "",
      //       name: "ddzs_index",
      //       component: () => import("@/views/dddx/ddzs/index.vue"),
      //       meta: { title: "督导综述" },
      //     },
      //     {
      //       path: "detail/:id",
      //       hidden: true,
      //       name: "dddx_jyj_detail",
      //       component: () => import("@/views/dddx/ddzs/detail/jyjDetail.vue"),
      //       meta: { title: "详情" },
      //       props: true,
      //     },
      //     {
      //       path: "detail/:id",
      //       hidden: true,
      //       name: "dddx_xxyzb_detail",
      //       component: () => import("@/views/dddx/ddzs/detail/xxyzbDetail.vue"),
      //       meta: { title: "详情" },
      //       props: true,
      //     },
      //     {
      //       path: "detail/:id",
      //       hidden: true,
      //       name: "dddx_xxwzb_detail",
      //       component: () => import("@/views/dddx/ddzs/detail/xxwzbDetail.vue"),
      //       meta: { title: "详情" },
      //       props: true,
      //     },
      //   ],
      // },
      {
        path: "xnddpd",
        name: "dddx-xnddpd",
        component: () => import("@/views/dddx/xnddpd/index.vue"),
        meta: { title: "学年督导评定" },
      },
      {
        path: "bbtj",
        name: "dddx-bbtj",
        component: () => import("@/views/dddx/bbtj/index.vue"),
        redirect: "/dddx/bbtj/bbtj-main",
        meta: { title: "报表统计" },
        children: [
          {
            hidden: true,
            path: "bbtj-main",
            name: "dddx-bbtj-main",
            component: () => import("@/views/dddx/bbtj/components/bbtj.vue"),
            meta: {
              title: "",
              checkList: ["show", "add", "edit", "delete", "detail", "export"],
            },
          },
          {
            hidden: true,
            path: "bbtj-detail/:id",
            name: "dddx-bbtj-detail",
            component: () => import("@/views/dddx/bbtj/components/detail.vue"),
            meta: {
              title: "详情",
              checkList: ["show", "edit", "export"],
            },
          },
        ],
      },
      // {
      //   path: "gzzj",
      //   name: "gzzj",
      //   component: () => import("@/views/dddx/gzzj/index.vue"),
      //   meta: { title: "工作总结" },
      // },
    ],
  },
  {
    path: "/zzjgyqx",
    component: AppLayout,
    name: "zzjgyqx",
    meta: { title: "", icon: "el-icon-document" },
    children: [
      {
        path: "",
        name: "zzjgyqx_main",
        // component: () => import("@/views/zzjgyqx/index.vue"),
        component: () => import("@/views/xmgl/qxfp/index.vue"),
        meta: { title: "组织机构与权限" },
      },
    ],
  },
  {
    path: "/xxzx",
    name: "xxzx_main",
    component: AppLayout,
    hidden: true,
    children: [
      {
        path: "",
        name: "xxzx",
        component: () => import("@/views/xxzx/index.vue"),
        meta: { title: "消息中心" },
      },
    ],
  },
  ...xxgzjcRoutes,
  ...xxbgOld,
  // 404 page must be placed at the end !!!
  { path: "*", redirect: "/404", hidden: true },
];

// function hasPermission(route, role) {
//   if (Array.isArray(route?.meta?.roles)) {
//     return route.meta.roles.includes(role);
//   } else {
//     return true;
//   }
// }

// function handleFilterRoutes(routes, role) {
//   const res = [];
//   routes.forEach((route) => {
//     const tmp = { ...route };
//     if (hasPermission(tmp, role)) {
//       if (tmp.children) {
//         tmp.children = handleFilterRoutes(tmp.children, role);
//       }
//       res.push(tmp);
//     }
//   });
//   return res;
// }

const _routes_ = await request({
  url: '/user/org_menu/setting_tyyx',
  data: {
    cpm_id: '80011',
    dept_id: userInfo.uo_id,
    keyname: "cmu_name"
  }
}).then(setting => {
  const requestRoutes = setting.data.cmu_name ?? {}
  // console.log('requestRoutes :>> ', requestRoutes);
  const requestRoutesKeyMap = {}
  Object.keys(requestRoutes).forEach(k => {
    const val = requestRoutes[k]
    if (val?.split(',')?.includes('show')) {
      requestRoutesKeyMap[k] = true
    }
  })
  // console.log('requestRoutesKeyMap :>> ', requestRoutesKeyMap);

  const filteredRoutesMap = {}
  // 维护路由树结构
  const RoutesTreeRelationMap = {}

  // const leafRouteMap: any = {}
  function initRoutesTreeRelation(routes, parent) {
    routes.forEach(route => {
      RoutesTreeRelationMap[route.name] = {
        parent,
        children: route.children === undefined ? undefined : initRoutesTreeRelation(route.children, [...parent, route.name])
      }
      if (route.children === undefined) {
        // leafRouteMap[route.name] = true
        if (requestRoutesKeyMap[route.name]) {
          filteredRoutesMap[route.name] = true
          RoutesTreeRelationMap[route.name].parent.forEach((k) => filteredRoutesMap[k] = true)
        }
      }
    })
  }

  initRoutesTreeRelation(allRoutes, [])
  // console.log('filteredRoutesMap :>> ', filteredRoutesMap);
  // export const RoutesCheck: any = {}
  function handleFilterRoutes(routes) {
    const res = []
    routes.forEach((route) => {
      const tmp = { ...route }
      // console.log('route :>> ', route, filteredRoutesMap[tmp.name]);

      if (filteredRoutesMap[tmp.name]) {
        if (!isAdmin) {
          tmp.meta.checkList = requestRoutes[tmp.name]?.split(',')
        }
        // RoutesCheck[tmp.name] = tmp.meta.checkList
        if (tmp.children) {
          tmp.children = handleFilterRoutes(tmp.children)
        }
        res.push(tmp)
      }
      // else if(!tmp?.meta?.checkList){
      //   if (tmp.children) {
      //     tmp.children = handleFilterRoutes(tmp.children)
      //   }
      //   res.push(tmp)
      // }
    })
    return res
  }

  const $routes = (isAdmin ? allRoutes : handleFilterRoutes(allRoutes)).sort((a, b) => a?.meta?.sort - b?.meta?.sort)
  let routes
  if ($routes?.length > 0)
    routes = $routes.concat([
      {
        path: "/404",
        component: () => import("@/views/404.vue"),
        hidden: true,
      },
      {
        path: "/",
        redirect: { name: $routes?.[0]?.name },
      }
    ])
  else {
    routes = [
      {
        path: '*',
        component: () => import('@/views/noSet.vue')
      }
    ]
  }
  sessionStorage.setItem('routes', JSON.stringify(routes))

  return routes
})

const createRouter = () =>
  new Router({
    // mode: 'history', // require service support
    scrollBehavior: () => ({ y: 0 }),
    routes: _routes_
    // routes: allRoutes,
  });

const router = createRouter();
//
// // Detail see: https://github.com/vuejs/vue-router/issues/1234#issuecomment-357941465
// export function resetRouter() {
//     const newRouter = createRouter();
//     router.matcher = newRouter.matcher; // reset router
// }

export default router;

/*
=====================================================================================
=====================================================================================
*/

import getPageTitle from "@/utils/get-page-title";

router.beforeEach(async (to, from, next) => {
  document.title = getPageTitle(to.meta.title);
  console.groupCollapsed(
    `router ${from.meta.title || "新进入"} => ${to.meta.title}`
  );
  console.log("from :>> ", from);
  console.log("to :>> ", to);
  console.groupEnd();
  next();
});
